=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-03-16 11:40:05 +0000
+++ loggerhead/apps/branch.py	2011-06-27 14:02:26 +0000
@@ -37,7 +37,7 @@
 from loggerhead.controllers.filediff_ui import FileDiffUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
-from loggerhead.controllers.revlog_ui import RevLogUI
+from loggerhead.controllers.revlog_ui import RevLogUI, RevLogJSONUI
 from loggerhead.controllers.search_ui import SearchUI
 from loggerhead.history import History
 from loggerhead import util
@@ -128,6 +128,10 @@
         'view': ViewUI,
         }
 
+    json_controllers_dict = {
+        '+revlog': RevLogJSONUI,
+        }
+
     def last_updated(self):
         h = self.get_history()
         change = h.get_changes([h.last_revid])[0]
@@ -164,7 +168,11 @@
                 self.absolute_url('/changes'))
         if path == 'static':
             return static_app(environ, start_response)
-        cls = self.controllers_dict.get(path)
+        elif path == '+json':
+            path2 = request.path_info_pop(environ)
+            cls = self.json_controllers_dict.get(path2)
+        else:
+            cls = self.controllers_dict.get(path)
         if cls is None:
             raise httpexceptions.HTTPNotFound()
         self.branch.lock_read()

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2011-03-16 11:40:05 +0000
+++ loggerhead/controllers/__init__.py	2011-06-27 14:02:26 +0000
@@ -67,8 +67,7 @@
         self.__history = self._history_callable()
         return self.__history
 
-    def __call__(self, environ, start_response):
-        z = time.time()
+    def get_core_values(self, environ):
         kwargs = dict(parse_querystring(environ))
         util.set_context(kwargs)
         args = []
@@ -83,6 +82,13 @@
             path = unicode('/'.join(args[1:]), 'utf-8')
         self.args = args
 
+        headers = {}
+        return self.get_values(path, kwargs, headers), headers
+
+    def __call__(self, environ, start_response):
+        z = time.time()
+        core_values, headers = self.get_core_values(environ)
+
         vals = {
             'static_url': self._branch.static_url,
             'branch': self._branch,
@@ -90,9 +96,8 @@
             'url': self._branch.context_url,
         }
         vals.update(templatefunctions)
-        headers = {}
 
-        vals.update(self.get_values(path, kwargs, headers))
+        vals.update(core_values)
 
         self.log.info('Getting information for %s: %.3f secs' % (
             self.__class__.__name__, time.time() - z))

=== modified file 'loggerhead/controllers/revlog_ui.py'
--- loggerhead/controllers/revlog_ui.py	2009-03-19 00:44:05 +0000
+++ loggerhead/controllers/revlog_ui.py	2011-06-27 14:02:26 +0000
@@ -1,7 +1,10 @@
+import datetime
+import simplejson
+import time
 import urllib
 
 from loggerhead import util
-from loggerhead.controllers import TemplatedBranchView
+from loggerhead.controllers import BufferingWriter, TemplatedBranchView
 
 
 class RevLogUI(TemplatedBranchView):
@@ -25,3 +28,46 @@
             'revid': revid,
             'url': self._branch.context_url,
         }
+
+
+class RevLogJSONUI(RevLogUI):
+
+    def get_json_values(self, environ):
+        core_values, headers = self.get_core_values(environ)
+        del core_values['branch'], core_values['util'], core_values['url']
+#        print core_values
+#        core_values['entry'] = core_values['entry']._properties
+#        core_values['file_changes'] = core_values['file_changes']._properties
+
+        return core_values, headers
+
+    def __call__(self, environ, start_response):
+        z = time.time()
+        json_values, headers = self.get_json_values(environ)
+
+        # XXX de-dupe this code.
+        self.log.info('Getting information for %s: %.3f secs' % (
+            self.__class__.__name__, time.time() - z))
+        #headers['Content-Type'] = 'application/json'
+        headers['Content-Type'] = 'text/plain'
+        writer = start_response("200 OK", headers.items())
+        if environ.get('REQUEST_METHOD') == 'HEAD':
+            # No content for a HEAD request
+            return []
+        z = time.time()
+        w = BufferingWriter(writer, 8192)
+        def encode_stuff(obj):
+            if isinstance(obj, util.Container):
+                d = obj.__dict__.copy()
+                del d['_properties']
+                return d
+            elif isinstance(obj, datetime.datetime):
+                return tuple(obj.utctimetuple())
+            raise TypeError(repr(obj) + " is not JSON serializable")
+
+        w.write(simplejson.dumps(json_values, default=encode_stuff))
+        w.flush()
+        self.log.info(
+            'Rendering %s: %.3f secs, %s bytes' % (
+                self.__class__.__name__, time.time() - z, w.bytes))
+        return []

